@@ -1,5 +1,10 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
+
 #include "evas_common.h"
 #include "evas_private.h"
+#include "../engines/common/evas_convert_color.h"
 
 /* private magic number for gradient objects */
 static const char o_type[] = "gradient";
@@ -44,6 +49,10 @@ static void evas_object_gradient_free(Evas_Object *obj);
 static void evas_object_gradient_render_pre(Evas_Object *obj);
 static void evas_object_gradient_render_post(Evas_Object *obj);
 
+static unsigned int evas_object_gradient_id_get(Evas_Object *obj);
+static unsigned int evas_object_gradient_visual_id_get(Evas_Object *obj);
+static void *evas_object_gradient_engine_data_get(Evas_Object *obj);
+
 static int evas_object_gradient_is_opaque(Evas_Object *obj);
 static int evas_object_gradient_was_opaque(Evas_Object *obj);
 
@@ -54,6 +63,9 @@ static const Evas_Object_Func object_func =
      evas_object_gradient_render,
      evas_object_gradient_render_pre,
      evas_object_gradient_render_post,
+     evas_object_gradient_id_get,
+     evas_object_gradient_visual_id_get,
+     evas_object_gradient_engine_data_get,
    /* these are optional. NULL = nothing */
      NULL,
      NULL,
@@ -99,6 +111,7 @@ evas_object_gradient_add(Evas *e)
 
 	o->engine_data = e->engine.func->gradient_new(e->engine.data.output);
      }
+   evas_object_change(obj);
    return obj;
 }
 
@@ -334,7 +347,7 @@ evas_object_gradient_fill_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas
  * @ingroup Evas_Object_Gradient_Fill_Group
  */
 EAPI void
-evas_object_gradient_fill_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
+evas_object_gradient_fill_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
 {
    Evas_Object_Gradient *o;
 
@@ -393,7 +406,7 @@ evas_object_gradient_fill_angle_set(Evas_Object *obj, Evas_Angle angle)
  * @ingroup Evas_Object_Gradient_Group
  */
 EAPI Evas_Angle
-evas_object_gradient_fill_angle_get(Evas_Object *obj)
+evas_object_gradient_fill_angle_get(const Evas_Object *obj)
 {
    Evas_Object_Gradient *o;
 
@@ -440,7 +453,7 @@ evas_object_gradient_fill_spread_set(Evas_Object *obj, int spread)
  * @ingroup Evas_Object_Gradient_Group
  */
 EAPI int
-evas_object_gradient_fill_spread_get(Evas_Object *obj)
+evas_object_gradient_fill_spread_get(const Evas_Object *obj)
 {
    Evas_Object_Gradient *o;
 
@@ -489,7 +502,7 @@ evas_object_gradient_angle_set(Evas_Object *obj, Evas_Angle angle)
  * @ingroup Evas_Object_Gradient_Group
  */
 EAPI Evas_Angle
-evas_object_gradient_angle_get(Evas_Object *obj)
+evas_object_gradient_angle_get(const Evas_Object *obj)
 {
    Evas_Object_Gradient *o;
 
@@ -534,7 +547,7 @@ evas_object_gradient_offset_set(Evas_Object *obj, float offset)
  * @ingroup Evas_Object_Gradient_Group
  */
 EAPI float
-evas_object_gradient_offset_get(Evas_Object *obj)
+evas_object_gradient_offset_get(const Evas_Object *obj)
 {
    Evas_Object_Gradient *o;
 
@@ -580,7 +593,7 @@ evas_object_gradient_direction_set(Evas_Object *obj, int direction)
  * @ingroup Evas_Object_Gradient_Group
  */
 EAPI int
-evas_object_gradient_direction_get(Evas_Object *obj)
+evas_object_gradient_direction_get(const Evas_Object *obj)
 {
    Evas_Object_Gradient *o;
 
@@ -685,7 +698,7 @@ evas_object_gradient_type_set(Evas_Object *obj, const char *name, const char *pa
  * @ingroup Evas_Object_Gradient_Type_Group
  */
 EAPI void
-evas_object_gradient_type_get(Evas_Object *obj, char **name, char **params)
+evas_object_gradient_type_get(const Evas_Object *obj, char **name, char **params)
 {
    Evas_Object_Gradient *o;
 
@@ -785,14 +798,13 @@ evas_object_gradient_init(Evas_Object *obj)
    obj->cur.geometry.h = 0;
    obj->cur.layer = 0;
    obj->cur.anti_alias = 1;
-   obj->cur.interpolation.color_space = EVAS_COLOR_SPACE_ARGB;
+   obj->cur.interpolation_color_space = EVAS_COLOR_SPACE_ARGB;
    obj->cur.render_op = EVAS_RENDER_BLEND;
    /* set up object-specific settings */
    obj->prev = obj->cur;
    /* set up methods (compulsory) */
    obj->func = &object_func;
    obj->type = o_type;
-   obj->changed = 1;
 }
 
 static void *
@@ -902,7 +914,7 @@ evas_object_gradient_render(Evas_Object *obj, void *output, void *context, void
 static void
 evas_object_gradient_render_pre(Evas_Object *obj)
 {
-   Evas_List *updates = NULL;
+   Evas_Rectangles rects = { 0, 0, NULL };
    Evas_Object_Gradient *o;
    int is_v, was_v;
 
@@ -933,7 +945,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
        (obj->cur.cache.clip.b != obj->prev.cache.clip.b) ||
        (obj->cur.cache.clip.a != obj->prev.cache.clip.a)))
      { o->gradient_changed = 1;  o->changed = 1; }
-   if (!o->gradient_changed && (obj->cur.interpolation.color_space != obj->prev.interpolation.color_space))
+   if (!o->gradient_changed && (obj->cur.interpolation_color_space != obj->prev.interpolation_color_space))
      { o->gradient_changed = 1;  o->changed = 1; }
    if (!o->changed && (obj->cur.render_op != obj->prev.render_op))
 	o->changed = 1;
@@ -973,7 +985,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
 								obj->cur.cache.clip.b, obj->cur.cache.clip.a);
 	obj->layer->evas->engine.func->context_color_interpolation_set(obj->layer->evas->engine.data.output,
 									obj->layer->evas->engine.data.context,
-									obj->cur.interpolation.color_space);
+									obj->cur.interpolation_color_space);
 	if (o->gradient_changed)
 	    obj->layer->evas->engine.func->gradient_render_pre(obj->layer->evas->engine.data.output,
 								obj->layer->evas->engine.data.context,
@@ -993,17 +1005,17 @@ evas_object_gradient_render_pre(Evas_Object *obj)
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	updates = evas_object_render_pre_visible_change(updates, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&rects, obj, is_v, was_v);
 	goto done;
      }
    /* its not visible - we accounted for it appearing or not so just abort */
    if (!is_v) goto done;
    /* clipper changed this is in addition to anything else for obj */
-   updates = evas_object_render_pre_clipper_change(updates, obj);
+   evas_object_render_pre_clipper_change(&rects, obj);
    /* gradient changed */
    if (o->changed || obj->restack)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* if it changed geometry */
@@ -1012,7 +1024,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
        (obj->cur.geometry.w != obj->prev.geometry.w) ||
        (obj->cur.geometry.h != obj->prev.geometry.h))
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* it obviously didn't change - add a NO obscure - this "unupdates"  this */
@@ -1028,7 +1040,7 @@ evas_object_gradient_render_pre(Evas_Object *obj)
 							    obj->cur.cache.clip.h);
    
    done:
-   evas_object_render_pre_effect_updates(updates, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&rects, obj, is_v, was_v);
 }
 
 static void
@@ -1051,7 +1063,6 @@ evas_object_gradient_render_post(Evas_Object *obj)
      }
    /* move cur to prev safely for object data */
    obj->prev = obj->cur;
-   obj->changed = 0;
    if (o->prev.type.name && (o->prev.type.name != o->cur.type.name))
 	free(o->prev.type.name);
    if (o->prev.type.params && (o->prev.type.params != o->cur.type.params))
@@ -1062,6 +1073,33 @@ evas_object_gradient_render_post(Evas_Object *obj)
    o->type_changed = 0;
 }
 
+static unsigned int evas_object_gradient_id_get(Evas_Object *obj)
+{
+   Evas_Object_Gradient *o;
+
+   o = (Evas_Object_Gradient *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_GRADIENT;
+}
+
+static unsigned int evas_object_gradient_visual_id_get(Evas_Object *obj)
+{
+   Evas_Object_Gradient *o;
+
+   o = (Evas_Object_Gradient *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_GRADIENT;
+}
+
+static void *evas_object_gradient_engine_data_get(Evas_Object *obj)
+{
+   Evas_Object_Gradient *o;
+
+   o = (Evas_Object_Gradient *)(obj->object_data);
+   if (!o) return NULL;
+   return o->engine_data;
+}
+
 static int
 evas_object_gradient_is_opaque(Evas_Object *obj)
 {
