@@ -1,5 +1,12 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
+
 #include "evas_common.h"
 #include "evas_private.h"
+#include "../engines/common/evas_convert_color.h"
+#include "../engines/common/evas_convert_colorspace.h"
+#include "../engines/common/evas_convert_yuv.h"
 
 /* private magic number for image objects */
 static const char o_type[] = "image";
@@ -12,9 +19,9 @@ struct _Evas_Object_Image
    DATA32            magic;
 
    struct {
-      struct {
-	 Evas_Coord  x, y, w, h;
-      } fill;
+      Evas_Common_Transform  transform;
+      int         spread;
+      Evas_Coord_Rectangle fill;
       struct {
 	 short       w, h, stride;
       } image;
@@ -65,10 +72,16 @@ static void evas_object_image_free(Evas_Object *obj);
 static void evas_object_image_render_pre(Evas_Object *obj);
 static void evas_object_image_render_post(Evas_Object *obj);
 
+static unsigned int evas_object_image_id_get(Evas_Object *obj);
+static unsigned int evas_object_image_visual_id_get(Evas_Object *obj);
+static void *evas_object_image_engine_data_get(Evas_Object *obj);
+
 static int evas_object_image_is_opaque(Evas_Object *obj);
 static int evas_object_image_was_opaque(Evas_Object *obj);
 static int evas_object_image_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 
+static void *evas_object_image_data_convert_internal(Evas_Object_Image *o, void *data, Evas_Colorspace to_cspace);
+
 static const Evas_Object_Func object_func =
 {
    /* methods (compulsory) */
@@ -76,6 +89,9 @@ static const Evas_Object_Func object_func =
    evas_object_image_render,
    evas_object_image_render_pre,
    evas_object_image_render_post,
+   evas_object_image_id_get,
+   evas_object_image_visual_id_get,
+   evas_object_image_engine_data_get,
    /* these are optional. NULL = nothing */
    NULL,
    NULL,
@@ -290,7 +306,7 @@ evas_object_image_file_set(Evas_Object *obj, const char *file, const char *key)
  * @ingroup Evas_Object_Image_File_Group
  */
 EAPI void
-evas_object_image_file_get(Evas_Object *obj, const char **file, const char **key)
+evas_object_image_file_get(const Evas_Object *obj, const char **file, const char **key)
 {
    Evas_Object_Image *o;
 
@@ -376,7 +392,7 @@ evas_object_image_border_set(Evas_Object *obj, int l, int r, int t, int b)
  * @ingroup Evas_Object_Image_Border_Group
  */
 EAPI void
-evas_object_image_border_get(Evas_Object *obj, int *l, int *r, int *t, int *b)
+evas_object_image_border_get(const Evas_Object *obj, int *l, int *r, int *t, int *b)
 {
    Evas_Object_Image *o;
 
@@ -452,7 +468,7 @@ evas_object_image_border_center_fill_set(Evas_Object *obj, Evas_Bool fill)
  * @ingroup Evas_Object_Image_Fill_Group
  */
 EAPI Evas_Bool
-evas_object_image_border_center_fill_get(Evas_Object *obj)
+evas_object_image_border_center_fill_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -527,7 +543,7 @@ evas_object_image_fill_set(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Co
  * @ingroup Evas_Object_Image_Fill_Group
  */
 EAPI void
-evas_object_image_fill_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
+evas_object_image_fill_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
 {
    Evas_Object_Image *o;
 
@@ -552,6 +568,132 @@ evas_object_image_fill_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_
    if (h) *h = o->cur.fill.h;
 }
 
+
+/**
+ * Sets the tiling mode for the given evas image object's fill.
+ * @param   obj   The given evas image object.
+ * @param   spread One of EVAS_TEXTURE_REFLECT, EVAS_TEXTURE_REPEAT,
+ * EVAS_TEXTURE_RESTRICT, or EVAS_TEXTURE_PAD.
+ * @ingroup Evas_Object_Image_Fill_Group
+ */
+EAPI void
+evas_object_image_fill_spread_set(Evas_Object *obj, int spread)
+{
+   Evas_Object_Image *o;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return;
+   MAGIC_CHECK_END();
+   if (spread == o->cur.spread) return;
+   o->cur.spread = spread;
+   o->changed = 1;
+   evas_object_change(obj);
+}
+
+/**
+ * Retrieves the spread (tiling mode) for the given image object's fill.
+ * @param   obj The given evas image object.
+ * @return  The current spread mode of the image object.
+ * @ingroup Evas_Object_Image_Fill_Group
+ */
+EAPI int
+evas_object_image_fill_spread_get(const Evas_Object *obj)
+{
+   Evas_Object_Image *o;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return EVAS_TEXTURE_REPEAT;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return EVAS_TEXTURE_REPEAT;
+   MAGIC_CHECK_END();
+   return o->cur.spread;
+}
+
+EAPI void
+evas_object_image_fill_transform_set (Evas_Object *obj, Evas_Transform *t)
+{
+   Evas_Object_Image *o;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return;
+   MAGIC_CHECK_END();
+   if (!t)
+     {
+	o->cur.transform.mxx = 1;
+	o->cur.transform.mxy = 0;
+	o->cur.transform.mxz = 0;
+	o->cur.transform.myx = 0;
+	o->cur.transform.myy = 1;
+	o->cur.transform.myz = 0;
+	o->cur.transform.mzx = 0;
+	o->cur.transform.mzy = 0;
+	o->cur.transform.mzz = 1;
+
+	o->changed;
+	evas_object_change(obj);
+	return;
+     }
+   if ( (o->cur.transform.mxx == t->mxx) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) ||
+	 (o->cur.transform.mxy == t->mxy) )
+	    return;
+
+   o->cur.transform.mxx = t->mxx;
+   o->cur.transform.mxy = t->mxy;
+   o->cur.transform.mxz = t->mxz;
+   o->cur.transform.myx = t->myx;
+   o->cur.transform.myy = t->myy;
+   o->cur.transform.myz = t->myz;
+   o->cur.transform.mzx = t->mzx;
+   o->cur.transform.mzy = t->mzy;
+   o->cur.transform.mzz = t->mzz;
+
+   o->changed;
+   evas_object_change(obj);
+}
+
+EAPI void
+evas_object_image_fill_transform_get (const Evas_Object *obj, Evas_Transform *t)
+{
+   Evas_Object_Image *o;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return;
+   MAGIC_CHECK_END();
+   if (t)
+     {
+        t->mxx = o->cur.transform.mxx;
+        t->mxy = o->cur.transform.mxy;
+        t->mxz = o->cur.transform.mxz;
+        t->myx = o->cur.transform.myx;
+        t->myy = o->cur.transform.myy;
+        t->myz = o->cur.transform.myz;
+        t->mzx = o->cur.transform.mzx;
+        t->mzy = o->cur.transform.mzy;
+        t->mzz = o->cur.transform.mzz;
+     }
+}
+
 /**
  * @defgroup Evas_Object_Image_Size Image Object Image Size Functions
  *
@@ -631,7 +773,7 @@ evas_object_image_size_set(Evas_Object *obj, int w, int h)
  * @ingroup Evas_Object_Image_Size
  */
 EAPI void
-evas_object_image_size_get(Evas_Object *obj, int *w, int *h)
+evas_object_image_size_get(const Evas_Object *obj, int *w, int *h)
 {
    Evas_Object_Image *o;
 
@@ -661,7 +803,7 @@ evas_object_image_size_get(Evas_Object *obj, int *w, int *h)
  * @ingroup Evas_Object_Image_Size
  */
 EAPI int
-evas_object_image_stride_get(Evas_Object *obj)
+evas_object_image_stride_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -686,7 +828,7 @@ evas_object_image_stride_get(Evas_Object *obj)
  * @ingroup Evas_Object_Image
  */
 EAPI int
-evas_object_image_load_error_get(Evas_Object *obj)
+evas_object_image_load_error_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -707,6 +849,46 @@ evas_object_image_load_error_get(Evas_Object *obj)
  * image object.
  */
 
+/**
+ * Converts the raw image data of the given image object to the
+ * specified colorspace.
+ *
+ * Note that this function does not modify the raw image data.
+ * If the requested colorspace is the same as the image colorspace
+ * nothing is done and NULL is returned. You should use
+ * evas_object_image_colorspace_get() to check the current image
+ * colorspace.
+ *
+ * See @ref evas_object_image_colorspace_get.
+ *
+ * @param obj The given image object.
+ * @param to_cspace The colorspace to which the image raw data will be converted.
+ * @return data A newly allocated data in the format specified by to_cspace.
+ * @ingroup Evas_Object_Image_Data
+ */
+EAPI void *
+evas_object_image_data_convert(Evas_Object *obj, Evas_Colorspace to_cspace)
+{
+   Evas_Object_Image *o;
+   DATA32 *data;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return NULL;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return NULL;
+   MAGIC_CHECK_END();
+   if (!o->engine_data) return NULL;
+   if (!o->cur.cspace == to_cspace) return NULL;
+   data = NULL;
+   o->engine_data = obj->layer->evas->engine.func->image_data_get(obj->layer->evas->engine.data.output,
+								  o->engine_data,
+								  0,
+								  &data);
+   return evas_object_image_data_convert_internal(o, data, to_cspace);
+}
+
 /**
  * Sets the raw image data of the given image object.
  *
@@ -788,7 +970,7 @@ evas_object_image_data_set(Evas_Object *obj, void *data)
  * @ingroup Evas_Object_Image_Data
  */
 EAPI void *
-evas_object_image_data_get(Evas_Object *obj, Evas_Bool for_writing)
+evas_object_image_data_get(const Evas_Object *obj, Evas_Bool for_writing)
 {
    Evas_Object_Image *o;
    DATA32 *data;
@@ -815,6 +997,41 @@ evas_object_image_data_get(Evas_Object *obj, Evas_Bool for_writing)
    return data;
 }
 
+/**
+ * Preload image in the background
+ *
+ * This function request the preload of the data image in the background. The
+ * worked is queued before being processed.
+ *
+ * If cancel is set, it will remove the image from the workqueue.
+ *
+ * @param obj The given image object.
+ * @param cancel 0 means add to the workqueue, 1 remove it.
+ * @ingroup Evas_Object_Image_Data
+ */
+EAPI void
+evas_object_image_preload(const Evas_Object *obj, Evas_Bool cancel)
+{
+   Evas_Object_Image *o;
+   DATA32 *data;
+
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return ;
+   MAGIC_CHECK_END();
+   o = (Evas_Object_Image *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Image, MAGIC_OBJ_IMAGE);
+   return ;
+   MAGIC_CHECK_END();
+   if (!o->engine_data) return ;
+   if (cancel)
+     obj->layer->evas->engine.func->image_data_preload_cancel(obj->layer->evas->engine.data.output,
+							      o->engine_data);
+   else
+     obj->layer->evas->engine.func->image_data_preload_request(obj->layer->evas->engine.data.output,
+							       o->engine_data,
+							       obj);
+}
+
 /**
  * Replaces the raw image data of the given image object.
  *
@@ -961,7 +1178,7 @@ evas_object_image_alpha_set(Evas_Object *obj, Evas_Bool has_alpha)
  * @ingroup Evas_Object_Image_Alpha
  */
 EAPI Evas_Bool
-evas_object_image_alpha_get(Evas_Object *obj)
+evas_object_image_alpha_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1022,7 +1239,7 @@ evas_object_image_smooth_scale_set(Evas_Object *obj, Evas_Bool smooth_scale)
  * @ingroup Evas_Object_Image_Scale
  */
 EAPI Evas_Bool
-evas_object_image_smooth_scale_get(Evas_Object *obj)
+evas_object_image_smooth_scale_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1083,7 +1300,7 @@ evas_object_image_reload(Evas_Object *obj)
  * @ingroup Evas_Object_Image
  */
 EAPI Evas_Bool
-evas_object_image_save(Evas_Object *obj, const char *file, const char *key, const char *flags)
+evas_object_image_save(const Evas_Object *obj, const char *file, const char *key, const char *flags)
 {
    Evas_Object_Image *o;
    DATA32 *data = NULL;
@@ -1121,18 +1338,29 @@ evas_object_image_save(Evas_Object *obj, const char *file, const char *key, cons
 	     else break;
 	  }
      }
-   im = evas_cache_image_empty(evas_common_image_cache_get());
+   im = (RGBA_Image*) evas_cache_image_data(evas_common_image_cache_get(),
+                                            o->cur.image.w,
+                                            o->cur.image.h,
+                                            data,
+                                            o->cur.has_alpha,
+                                            EVAS_COLORSPACE_ARGB8888);
    if (im)
      {
-	if (o->cur.has_alpha) im->flags |= RGBA_IMAGE_HAS_ALPHA;
+	if (o->cur.cspace == EVAS_COLORSPACE_ARGB8888)
+	  im->image.data = data;
+	else
+	  im->image.data = evas_object_image_data_convert_internal(o,
+								   data,
+								   EVAS_COLORSPACE_ARGB8888);
+	if (im->image.data)
+	  {
+	     ok = evas_common_save_image_to_file(im, file, key, quality, compress);
 
-        im->image->data = data;
-        im->image->w = o->cur.image.w;
-        im->image->h = o->cur.image.h;
-        im->image->no_free = 1;
-        ok = evas_common_save_image_to_file(im, file, key, quality, compress);
+	     if (o->cur.cspace != EVAS_COLORSPACE_ARGB8888)
+	       free(im->image.data);
+	  }
 
-	evas_cache_image_drop(im);
+	evas_cache_image_drop(&im->cache_entry);
      }
    return ok;
 }
@@ -1284,7 +1512,7 @@ evas_object_image_pixels_dirty_set(Evas_Object *obj, Evas_Bool dirty)
  * @ingroup Evas_Object_Image
  */
 EAPI Evas_Bool
-evas_object_image_pixels_dirty_get(Evas_Object *obj)
+evas_object_image_pixels_dirty_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1334,7 +1562,7 @@ evas_object_image_load_dpi_set(Evas_Object *obj, double dpi)
  *
  */
 EAPI double
-evas_object_image_load_dpi_get(Evas_Object *obj)
+evas_object_image_load_dpi_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1378,7 +1606,7 @@ evas_object_image_load_size_set(Evas_Object *obj, int w, int h)
 }
 
 EAPI void
-evas_object_image_load_size_get(Evas_Object *obj, int *w, int *h)
+evas_object_image_load_size_get(const Evas_Object *obj, int *w, int *h)
 {
    Evas_Object_Image *o;
 
@@ -1428,7 +1656,7 @@ evas_object_image_load_scale_down_set(Evas_Object *obj, int scale_down)
  *
  */
 EAPI int
-evas_object_image_load_scale_down_get(Evas_Object *obj)
+evas_object_image_load_scale_down_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1474,7 +1702,7 @@ evas_object_image_colorspace_set(Evas_Object *obj, Evas_Colorspace cspace)
  *
  */
 EAPI Evas_Colorspace
-evas_object_image_colorspace_get(Evas_Object *obj)
+evas_object_image_colorspace_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1518,7 +1746,7 @@ evas_object_image_native_surface_set(Evas_Object *obj, Evas_Native_Surface *surf
  *
  */
 EAPI Evas_Native_Surface *
-evas_object_image_native_surface_get(Evas_Object *obj)
+evas_object_image_native_surface_get(const Evas_Object *obj)
 {
    Evas_Object_Image *o;
 
@@ -1633,7 +1861,7 @@ evas_image_cache_set(Evas *e, int size)
  *
  */
 EAPI int
-evas_image_cache_get(Evas *e)
+evas_image_cache_get(const Evas *e)
 {
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return 0;
@@ -1795,6 +2023,8 @@ evas_object_image_new(void)
    o->cur.smooth_scale = 1;
    o->cur.border.fill = 1;
    o->cur.cspace = EVAS_COLORSPACE_ARGB8888;
+   o->cur.transform.mxx = o->cur.transform.myy = o->cur.transform.mzz = 1;
+   o->cur.spread = EVAS_TEXTURE_REPEAT;
    o->prev = o->cur;
    return o;
 }
@@ -2023,7 +2253,7 @@ evas_object_image_render(Evas_Object *obj, void *output, void *context, void *su
 static void
 evas_object_image_render_pre(Evas_Object *obj)
 {
-   Evas_List *updates = NULL;
+   Evas_Rectangles rects = { 0, 0, NULL };
    Evas_Object_Image *o;
    int is_v, was_v;
 
@@ -2049,17 +2279,17 @@ evas_object_image_render_pre(Evas_Object *obj)
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	updates = evas_object_render_pre_visible_change(updates, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&rects, obj, is_v, was_v);
 	if (!o->pixel_updates) goto done;
      }
    /* it's not visible - we accounted for it appearing or not so just abort */
    if (!is_v) goto done;
    /* clipper changed this is in addition to anything else for obj */
-   updates = evas_object_render_pre_clipper_change(updates, obj);
+   evas_object_render_pre_clipper_change(&rects, obj);
    /* if we restacked (layer or just within a layer) and don't clip anyone */
    if (obj->restack)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	if (!o->pixel_updates) goto done;
      }
    /* if it changed color */
@@ -2068,19 +2298,19 @@ evas_object_image_render_pre(Evas_Object *obj)
        (obj->cur.color.b != obj->prev.color.b) ||
        (obj->cur.color.a != obj->prev.color.a))
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	if (!o->pixel_updates) goto done;
      }
    /* if it changed render op */
    if (obj->cur.render_op != obj->prev.render_op)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	if (!o->pixel_updates) goto done;
      }
    /* if it changed anti_alias */
    if (obj->cur.anti_alias != obj->prev.anti_alias)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	if (!o->pixel_updates) goto done;
      }
    if (o->changed)
@@ -2091,7 +2321,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 	    ((!o->cur.key) && (o->prev.key))
 	    )
 	  {
-	     updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	     evas_object_render_pre_prev_cur_add(&rects, obj);
 	     if (!o->pixel_updates) goto done;
 	  }
 	if ((o->cur.image.w != o->prev.image.w) ||
@@ -2100,7 +2330,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 	    (o->cur.cspace != o->prev.cspace) ||
 	    (o->cur.smooth_scale != o->prev.smooth_scale))
 	  {
-	     updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	     evas_object_render_pre_prev_cur_add(&rects, obj);
 	     if (!o->pixel_updates) goto done;
 	  }
 	if ((o->cur.border.l != o->prev.border.l) ||
@@ -2108,12 +2338,12 @@ evas_object_image_render_pre(Evas_Object *obj)
 	    (o->cur.border.t != o->prev.border.t) ||
 	    (o->cur.border.b != o->prev.border.b))
 	  {
-	     updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	     evas_object_render_pre_prev_cur_add(&rects, obj);
 	     if (!o->pixel_updates) goto done;
 	  }
 	if (o->dirty_pixels)
 	  {
-	     updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	     evas_object_render_pre_prev_cur_add(&rects, obj);
 	     if (!o->pixel_updates) goto done;
 	  }
      }
@@ -2131,23 +2361,15 @@ evas_object_image_render_pre(Evas_Object *obj)
        (!o->pixel_updates)
        )
      {
-	Evas_Rectangle *r;
-	Evas_List *rl;
-
-	rl = evas_rects_return_difference_rects(obj->cur.geometry.x,
-						obj->cur.geometry.y,
-						obj->cur.geometry.w,
-						obj->cur.geometry.h,
-						obj->prev.geometry.x,
-						obj->prev.geometry.y,
-						obj->prev.geometry.w,
-						obj->prev.geometry.h);
-	while (rl)
-	  {
-	     r = rl->data;
-	     rl = evas_list_remove(rl, r);
-	     updates = evas_list_append(updates, r);
-	  }
+	evas_rects_return_difference_rects(&rects,
+					   obj->cur.geometry.x,
+					   obj->cur.geometry.y,
+					   obj->cur.geometry.w,
+					   obj->cur.geometry.h,
+					   obj->prev.geometry.x,
+					   obj->prev.geometry.y,
+					   obj->prev.geometry.w,
+					   obj->prev.geometry.h);
 	if (!o->pixel_updates) goto done;
      }
    if (((obj->cur.geometry.x != obj->prev.geometry.x) ||
@@ -2156,7 +2378,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 	(obj->cur.geometry.h != obj->prev.geometry.h))
        )
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	if (!o->pixel_updates) goto done;
      }
    if (o->changed)
@@ -2166,7 +2388,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 	    (o->cur.fill.w != o->prev.fill.w) ||
 	    (o->cur.fill.h != o->prev.fill.h))
 	  {
-	     updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	     evas_object_render_pre_prev_cur_add(&rects, obj);
 	     if (!o->pixel_updates) goto done;
 	  }
 	if ((o->cur.border.l == 0) &&
@@ -2176,7 +2398,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 	  {
 	     while (o->pixel_updates)
 	       {
-		  Evas_Rectangle *r, *rr;
+		  Evas_Rectangle *rr;
 		  Evas_Coord idw, idh, idx, idy;
 		  int x, y, w, h;
 
@@ -2200,16 +2422,18 @@ evas_object_image_render_pre(Evas_Object *obj)
 		       w = ((int)(idx + idw)) - x;
 		       while (idy < obj->cur.geometry.h)
 			 {
+			    Evas_Rectangle r;
+
 			    y = idy;
 			    h = ((int)(idy + idh)) - y;
-			    NEW_RECT(r, x, y, w, h);
-			    r->x = ((rr->x - 1) * r->w) / o->cur.image.w;
-			    r->y = ((rr->y - 1) * r->h) / o->cur.image.h;
-			    r->w = ((rr->w + 2) * r->w) / o->cur.image.w;
-			    r->h = ((rr->h + 2) * r->h) / o->cur.image.h;
-			    r->x += obj->cur.geometry.x + x;
-			    r->y += obj->cur.geometry.y + y;
-			    if (r) updates = evas_list_append(updates, r);
+
+			    r.x = ((rr->x - 1) * w) / o->cur.image.w;
+			    r.y = ((rr->y - 1) * h) / o->cur.image.h;
+			    r.w = ((rr->w + 2) * w) / o->cur.image.w;
+			    r.h = ((rr->h + 2) * h) / o->cur.image.h;
+			    r.x += obj->cur.geometry.x + x;
+			    r.y += obj->cur.geometry.y + y;
+			    evas_add_rect(&rects, r.x, r.y, r.w, r.h);
 			    idy += h;
 			 }
 		       idx += idw;
@@ -2232,7 +2456,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 		       free(r);
 		    }
 		  obj->layer->evas->engine.func->image_dirty_region(obj->layer->evas->engine.data.output, o->engine_data, 0, 0, o->cur.image.w, o->cur.image.h);
-		  updates = evas_object_render_pre_prev_cur_add(updates, obj);
+		  evas_object_render_pre_prev_cur_add(&rects, obj);
 		  goto done;
 	       }
 	  }
@@ -2248,7 +2472,7 @@ evas_object_image_render_pre(Evas_Object *obj)
 							    obj->cur.cache.clip.w,
 							    obj->cur.cache.clip.h);
    done:
-   evas_object_render_pre_effect_updates(updates, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&rects, obj, is_v, was_v);
 }
 
 static void
@@ -2284,6 +2508,33 @@ evas_object_image_render_post(Evas_Object *obj)
    /* FIXME: copy strings across */
 }
 
+static unsigned int evas_object_image_id_get(Evas_Object *obj)
+{
+   Evas_Object_Image *o;
+
+   o = (Evas_Object_Image *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_IMAGE;
+}
+
+static unsigned int evas_object_image_visual_id_get(Evas_Object *obj)
+{
+   Evas_Object_Image *o;
+
+   o = (Evas_Object_Image *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_IMAGE;
+}
+
+static void *evas_object_image_engine_data_get(Evas_Object *obj)
+{
+   Evas_Object_Image *o;
+
+   o = (Evas_Object_Image *)(obj->object_data);
+   if (!o) return NULL;
+   return o->engine_data;
+}
+
 static int
 evas_object_image_is_opaque(Evas_Object *obj)
 {
@@ -2332,7 +2583,7 @@ static int
 evas_object_image_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 {
    Evas_Object_Image *o;
-   void *data;
+   DATA32 *data;
    int w, h, stride;
    int a;
 
@@ -2354,7 +2605,7 @@ evas_object_image_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
    o->engine_data = obj->layer->evas->engine.func->image_data_get(obj->layer->evas->engine.data.output,
 								  o->engine_data,
 								  0,
-								  (DATA32**) &data);
+								  &data);
    if (!data)
      return 0;
 
@@ -2365,8 +2616,8 @@ evas_object_image_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 	  a = (*((DATA32*)(data)) >> 24) & 0xff;
 	  break;
 	case EVAS_COLORSPACE_RGB565_A5P:
-	  data = ((DATA16*)(data) + (h * stride));
-	  data = ((DATA8*)(data) + ((y * stride) + x));
+           data = (void*) ((DATA16*)(data) + (h * stride));
+	  data = (void*) ((DATA8*)(data) + ((y * stride) + x));
 	  a = (*((DATA8*)(data))) & 0x1f;
 	  break;
 	default:
@@ -2376,3 +2627,36 @@ evas_object_image_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
 
    return (a != 0);
 }
+
+static void *
+evas_object_image_data_convert_internal(Evas_Object_Image *o, void *data, Evas_Colorspace to_cspace)
+{
+   void *out = NULL;
+
+   if (!data)
+     return NULL;
+
+   switch (o->cur.cspace)
+     {
+	case EVAS_COLORSPACE_ARGB8888:
+	  out = evas_common_convert_argb8888_to(data,
+						o->cur.image.w,
+						o->cur.image.h,
+						o->cur.image.stride,
+						o->cur.has_alpha,
+						to_cspace);
+	  break;
+	case EVAS_COLORSPACE_RGB565_A5P:
+	  out = evas_common_convert_rgb565_a5p_to(data,
+						  o->cur.image.w,
+						  o->cur.image.h,
+						  o->cur.image.stride,
+						  o->cur.has_alpha,
+						  to_cspace);
+	  break;
+	default:
+	  break;
+     }
+
+   return out;
+}
