@@ -33,6 +33,10 @@ static void evas_object_line_free(Evas_Object *obj);
 static void evas_object_line_render_pre(Evas_Object *obj);
 static void evas_object_line_render_post(Evas_Object *obj);
 
+static unsigned int evas_object_line_id_get(Evas_Object *obj);
+static unsigned int evas_object_line_visual_id_get(Evas_Object *obj);
+static void *evas_object_line_engine_data_get(Evas_Object *obj);
+
 static int evas_object_line_is_opaque(Evas_Object *obj);
 static int evas_object_line_was_opaque(Evas_Object *obj);
 static int evas_object_line_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
@@ -46,6 +50,9 @@ static const Evas_Object_Func object_func =
      evas_object_line_render,
      evas_object_line_render_pre,
      evas_object_line_render_post,
+     evas_object_line_id_get,
+     evas_object_line_visual_id_get,
+     evas_object_line_engine_data_get,
    /* these are optional. NULL = nothing */
      NULL,
      NULL,
@@ -184,7 +191,7 @@ evas_object_line_xy_set(Evas_Object *obj, Evas_Coord x1, Evas_Coord y1, Evas_Coo
  * @ingroup Evas_Line_Group
  */
 EAPI void
-evas_object_line_xy_get(Evas_Object *obj, Evas_Coord *x1, Evas_Coord *y1, Evas_Coord *x2, Evas_Coord *y2)
+evas_object_line_xy_get(const Evas_Object *obj, Evas_Coord *x1, Evas_Coord *y1, Evas_Coord *x2, Evas_Coord *y2)
 {
    Evas_Object_Line *o;
 
@@ -298,7 +305,7 @@ evas_object_line_render(Evas_Object *obj, void *output, void *context, void *sur
 static void
 evas_object_line_render_pre(Evas_Object *obj)
 {
-   Evas_List *updates = NULL;
+   Evas_Rectangles rects = { 0, 0, NULL };
    Evas_Object_Line *o;
    int is_v, was_v;
 
@@ -324,29 +331,29 @@ evas_object_line_render_pre(Evas_Object *obj)
    was_v = evas_object_was_visible(obj);
    if (is_v != was_v)
      {
-	updates = evas_object_render_pre_visible_change(updates, obj, is_v, was_v);
+	evas_object_render_pre_visible_change(&rects, obj, is_v, was_v);
 	goto done;
      }
    /* it's not visible - we accounted for it appearing or not so just abort */
    if (!is_v) goto done;
    /* clipper changed this is in addition to anything else for obj */
-   updates = evas_object_render_pre_clipper_change(updates, obj);
+   evas_object_render_pre_clipper_change(&rects, obj);
    /* if we restacked (layer or just within a layer) */
    if (obj->restack)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* if it changed anti_alias */
    if (obj->cur.anti_alias != obj->prev.anti_alias)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* if it changed render op */
    if (obj->cur.render_op != obj->prev.render_op)
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* if it changed color */
@@ -355,7 +362,7 @@ evas_object_line_render_pre(Evas_Object *obj)
        (obj->cur.color.b != obj->prev.color.b) ||
        (obj->cur.color.a != obj->prev.color.a))
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    /* if it changed geometry - and obviously not visibility or color */
@@ -372,11 +379,11 @@ evas_object_line_render_pre(Evas_Object *obj)
 	 (o->cur.y2 != o->prev.y2)))
        )
      {
-	updates = evas_object_render_pre_prev_cur_add(updates, obj);
+	evas_object_render_pre_prev_cur_add(&rects, obj);
 	goto done;
      }
    done:
-   evas_object_render_pre_effect_updates(updates, obj, is_v, was_v);
+   evas_object_render_pre_effect_updates(&rects, obj, is_v, was_v);
 }
 
 static void
@@ -403,6 +410,33 @@ evas_object_line_render_post(Evas_Object *obj)
    o->changed = 0;
 }
 
+static unsigned int evas_object_line_id_get(Evas_Object *obj)
+{
+   Evas_Object_Line *o;
+
+   o = (Evas_Object_Line *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_LINE;
+}
+
+static unsigned int evas_object_line_visual_id_get(Evas_Object *obj)
+{
+   Evas_Object_Line *o;
+
+   o = (Evas_Object_Line *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_SHAPE;
+}
+
+static void *evas_object_line_engine_data_get(Evas_Object *obj)
+{
+   Evas_Object_Line *o;
+
+   o = (Evas_Object_Line *)(obj->object_data);
+   if (!o) return NULL;
+   return o->engine_data;
+}
+
 static int
 evas_object_line_is_opaque(Evas_Object *obj)
 {
