@@ -8,6 +8,12 @@ struct _Evas_Object_Smart
 {
    DATA32            magic;
    void             *engine_data;
+   void             *data;
+   Evas_List        *callbacks;
+   Evas_Object_List *contained;
+   int               walking_list;
+   Evas_Bool         deletions_waiting : 1;
+   Evas_Bool         need_recalculate : 1;
 };
 
 struct _Evas_Smart_Callback
@@ -27,6 +33,10 @@ static void evas_object_smart_free(Evas_Object *obj);
 static void evas_object_smart_render_pre(Evas_Object *obj);
 static void evas_object_smart_render_post(Evas_Object *obj);
 
+static unsigned int evas_object_smart_id_get(Evas_Object *obj);
+static unsigned int evas_object_smart_visual_id_get(Evas_Object *obj);
+static void *evas_object_smart_engine_data_get(Evas_Object *obj);
+
 static const Evas_Object_Func object_func =
 {
    /* methods (compulsory) */
@@ -34,6 +44,9 @@ static const Evas_Object_Func object_func =
      evas_object_smart_render,
      evas_object_smart_render_pre,
      evas_object_smart_render_post,
+     evas_object_smart_id_get,
+     evas_object_smart_visual_id_get,
+     evas_object_smart_engine_data_get,
      /* these are optional. NULL = nothing */
      NULL,
      NULL,
@@ -78,7 +91,7 @@ evas_object_smart_data_set(Evas_Object *obj, void *data)
    MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
    return;
    MAGIC_CHECK_END();
-   obj->smart.data = data;
+   o->data = data;
 }
 
 /**
@@ -90,7 +103,7 @@ evas_object_smart_data_set(Evas_Object *obj, void *data)
  * @ingroup Evas_Smart_Object_Group
  */
 EAPI void *
-evas_object_smart_data_get(Evas_Object *obj)
+evas_object_smart_data_get(const Evas_Object *obj)
 {
    Evas_Object_Smart *o;
 
@@ -101,7 +114,7 @@ evas_object_smart_data_get(Evas_Object *obj)
    MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
    return NULL;
    MAGIC_CHECK_END();
-   return obj->smart.data;
+   return o->data;
 }
 
 /**
@@ -112,7 +125,7 @@ evas_object_smart_data_get(Evas_Object *obj)
  * @ingroup Evas_Smart_Object_Group
  */
 EAPI Evas_Smart *
-evas_object_smart_smart_get(Evas_Object *obj)
+evas_object_smart_smart_get(const Evas_Object *obj)
 {
    Evas_Object_Smart *o;
 
@@ -177,7 +190,7 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
    obj->cur.layer = obj->layer->layer;
    obj->layer->usage++;
    obj->smart.parent = smart_obj;
-   smart_obj->smart.contained = evas_object_list_append(smart_obj->smart.contained, obj);
+   o->contained = evas_object_list_append(o->contained, obj);
    evas_object_smart_member_cache_invalidate(obj);
    obj->restack = 1;
    evas_object_change(obj);
@@ -197,12 +210,16 @@ evas_object_smart_member_add(Evas_Object *obj, Evas_Object *smart_obj)
 EAPI void
 evas_object_smart_member_del(Evas_Object *obj)
 {
+   Evas_Object_Smart *o;
+
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return;
    MAGIC_CHECK_END();
 
    if (!obj->smart.parent) return;
-   obj->smart.parent->smart.contained = evas_object_list_remove(obj->smart.parent->smart.contained, obj);
+
+   o = (Evas_Object_Smart *)(obj->smart.parent->object_data);
+   o->contained = evas_object_list_remove(o->contained, obj);
    obj->smart.parent = NULL;
    evas_object_smart_member_cache_invalidate(obj);
    obj->layer->usage--;
@@ -219,7 +236,7 @@ evas_object_smart_member_del(Evas_Object *obj)
  * @ingroup Evas_Smart_Object_Group
  */
 EAPI Evas_Object *
-evas_object_smart_parent_get(Evas_Object *obj)
+evas_object_smart_parent_get(const Evas_Object *obj)
 {
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
@@ -235,22 +252,36 @@ evas_object_smart_parent_get(Evas_Object *obj)
  * The returned list should be freed with evas_list_free() when you no longer need it
  */
 EAPI Evas_List *
-evas_object_smart_members_get(Evas_Object *obj)
+evas_object_smart_members_get(const Evas_Object *obj)
 {
+   Evas_Object_Smart *o;
    Evas_List *members;
    Evas_Object_List *member;
    
    MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
    return NULL;
    MAGIC_CHECK_END();
-   
+   o = (Evas_Object_Smart *)(obj->object_data);
+   MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
+   return NULL;
+   MAGIC_CHECK_END();
+
    members = NULL;
-   for (member = obj->smart.contained; member; member = member->next)
+   for (member = o->contained; member; member = member->next)
       members = evas_list_append(members, member);
    
    return members;
 }
 
+const Evas_Object_List *
+evas_object_smart_members_get_direct(const Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+   return o->contained;
+}
+
 /**
  * Instantiates a new smart object described by @p s.
  *
@@ -313,7 +344,7 @@ evas_object_smart_callback_add(Evas_Object *obj, const char *event, void (*func)
    cb->event = evas_stringshare_add(event);
    cb->func = func;
    cb->func_data = (void *)data;
-   obj->smart.callbacks = evas_list_prepend(obj->smart.callbacks, cb);
+   o->callbacks = evas_list_prepend(o->callbacks, cb);
 }
 
 /**
@@ -341,7 +372,7 @@ evas_object_smart_callback_del(Evas_Object *obj, const char *event, void (*func)
    return NULL;
    MAGIC_CHECK_END();
    if (!event) return NULL;
-   for (l = obj->smart.callbacks; l; l = l->next)
+   for (l = o->callbacks; l; l = l->next)
      {
 	Evas_Smart_Callback *cb;
 
@@ -352,7 +383,7 @@ evas_object_smart_callback_del(Evas_Object *obj, const char *event, void (*func)
 
 	     data = cb->func_data;
 	     cb->delete_me = 1;
-	     obj->smart.deletions_waiting = 1;
+	     o->deletions_waiting = 1;
 	     evas_object_smart_callbacks_clear(obj);
 	     return data;
 	  }
@@ -388,8 +419,8 @@ evas_object_smart_callback_call(Evas_Object *obj, const char *event, void *event
    MAGIC_CHECK_END();
    if (!event) return;
    if (obj->delete_me) return;
-   obj->smart.walking_list++;
-   for (l = obj->smart.callbacks; l; l = l->next)
+   o->walking_list++;
+   for (l = o->callbacks; l; l = l->next)
      {
 	Evas_Smart_Callback *cb;
 
@@ -402,19 +433,179 @@ evas_object_smart_callback_call(Evas_Object *obj, const char *event, void *event
 	if (obj->delete_me)
 	  break;
      }
-   obj->smart.walking_list--;
+   o->walking_list--;
    evas_object_smart_callbacks_clear(obj);
 }
 
+/**
+ * Set the need_recalculate flag of given smart object.
+ *
+ * If this flag is set then calculate() callback (method) of the given
+ * smart object will be called, if one is provided, during render phase
+ * usually evas_render(). After this step, this flag will be automatically
+ * unset.
+ *
+ * If no calculate() is provided, this flag will be left unchanged.
+ *
+ * @note just setting this flag will not make scene dirty and evas_render()
+ *       will have no effect. To do that, use evas_object_smart_changed(),
+ *       that will automatically call this function with 1 as parameter.
+ *
+ * @param obj the smart object
+ * @param value if one want to set or unset the need_recalculate flag.
+ *
+ * @ingroup Evas_Smart_Object_Group
+ */
+EAPI void
+evas_object_smart_need_recalculate_set(Evas_Object *obj, Evas_Bool value)
+{
+   Evas_Object_Smart *o;
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = obj->object_data;
+   MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
+   return;
+   MAGIC_CHECK_END();
+
+   value = !!value;
+   if (o->need_recalculate == value)
+     return;
+   o->need_recalculate = value;
+
+   if (!obj->smart.smart->smart_class->calculate)
+     return;
+
+   /* XXX: objects can be present multiple times in calculate_objects()
+    * XXX: after a set-unset-set cycle, but it's not a problem since
+    * XXX: on _evas_render_call_smart_calculate() will check for the flag
+    * XXX: and it will be unset after the first.
+    */
+   if (o->need_recalculate)
+     {
+	Evas *e;
+	e = obj->layer->evas;
+	_evas_array_append(&e->calculate_objects, obj);
+     }
+   /* TODO: else, remove from array */
+}
+
+/**
+ * Get the current value of need_recalculate flag.
+ *
+ * @note this flag will be unset during the render phase, after calculate()
+ *       is called if one is provided.  If no calculate() is provided, then
+ *       the flag will be left unchanged after render phase.
+ *
+ * @param obj the smart object
+ * @return if flag is set or not.
+ *
+ * @ingroup Evas_Smart_Object_Group
+ */
+EAPI Evas_Bool
+evas_object_smart_need_recalculate_get(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = obj->object_data;
+   MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
+   return;
+   MAGIC_CHECK_END();
+
+   return o->need_recalculate;
+}
+
+/**
+ * Call user provided calculate() and unset need_calculate.
+ *
+ * @param obj the smart object
+ * @return if flag is set or not.
+ *
+ * @ingroup Evas_Smart_Object_Group
+ */
+EAPI void
+evas_object_smart_calculate(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   o = obj->object_data;
+   MAGIC_CHECK(o, Evas_Object_Smart, MAGIC_OBJ_SMART);
+   return;
+   MAGIC_CHECK_END();
+
+   if (obj->smart.smart->smart_class->calculate)
+     obj->smart.smart->smart_class->calculate(obj);
+   o->need_recalculate = 0;
+}
+
+/**
+ * Call calculate() on all smart objects that need_recalculate.
+ *
+ * @internal
+ */
+void
+evas_call_smarts_calculate(Evas *e)
+{
+   Evas_Array *calculate;
+   unsigned int i;
+
+   calculate = &e->calculate_objects;
+   for (i = 0; i < calculate->count; ++i)
+     {
+	Evas_Object *obj;
+	Evas_Object_Smart *o;
+
+	obj = _evas_array_get(calculate, i);
+	if (obj->delete_me)
+	  continue;
+
+	o = obj->object_data;
+	if (o->need_recalculate)
+	  {
+	     obj->smart.smart->smart_class->calculate(obj);
+	     o->need_recalculate = 0;
+	  }
+     }
+
+   evas_array_flush(calculate);
+}
+
+/**
+ * Mark smart object as changed, dirty.
+ *
+ * This will inform the scene that it changed and needs to be redraw, also
+ * setting need_recalculate on the given object.
+ *
+ * @see evas_object_smart_need_recalculate_set().
+ *
+ * @ingroup Evas_Smart_Object_Group
+ */
+EAPI void
+evas_object_smart_changed(Evas_Object *obj)
+{
+   MAGIC_CHECK(obj, Evas_Object, MAGIC_OBJ);
+   return;
+   MAGIC_CHECK_END();
+   evas_object_change(obj);
+   evas_object_smart_need_recalculate_set(obj, 1);
+}
+
 /* internal calls */
 static void
 evas_object_smart_callbacks_clear(Evas_Object *obj)
 {
+   Evas_Object_Smart *o;
    Evas_List *l;
 
-   if (obj->smart.walking_list) return;
-   if (!obj->smart.deletions_waiting) return;
-   for (l = obj->smart.callbacks; l;)
+   o = (Evas_Object_Smart *)(obj->object_data);
+
+   if (o->walking_list) return;
+   if (!o->deletions_waiting) return;
+   for (l = o->callbacks; l;)
      {
 	Evas_Smart_Callback *cb;
 
@@ -422,7 +613,7 @@ evas_object_smart_callbacks_clear(Evas_Object *obj)
 	l = l->next;
 	if (cb->delete_me)
 	  {
-	     obj->smart.callbacks = evas_list_remove(obj->smart.callbacks, cb);
+	     o->callbacks = evas_list_remove(o->callbacks, cb);
 	     if (cb->event) evas_stringshare_del(cb->event);
 	     free(cb);
 	  }
@@ -444,34 +635,46 @@ evas_object_smart_del(Evas_Object *obj)
 void
 evas_object_smart_cleanup(Evas_Object *obj)
 {
-   Evas_Smart *s;
+   Evas_Object_Smart *o;
 
-   s = obj->smart.smart;
    if (obj->smart.parent)
      evas_object_smart_member_del(obj);
-   while (obj->smart.contained)
-     evas_object_smart_member_del((Evas_Object *)obj->smart.contained);
-   while (obj->smart.callbacks)
-     {
-	Evas_Smart_Callback *cb;
 
-	cb = obj->smart.callbacks->data;
-	obj->smart.callbacks = evas_list_remove(obj->smart.callbacks, cb);
-	if (cb->event) evas_stringshare_del(cb->event);
-	free(cb);
+   o = (Evas_Object_Smart *)(obj->object_data);
+   if (o->magic == MAGIC_OBJ_SMART)
+     {
+	while (o->contained)
+	  evas_object_smart_member_del((Evas_Object *)o->contained);
+
+	while (o->callbacks)
+	  {
+	     Evas_Smart_Callback *cb;
+
+	     cb = o->callbacks->data;
+	     o->callbacks = evas_list_remove(o->callbacks, cb);
+	     if (cb->event) evas_stringshare_del(cb->event);
+	     free(cb);
+	  }
+
+	o->data = NULL;
      }
+
    obj->smart.parent = NULL;
-   obj->smart.data = NULL;
    obj->smart.smart = NULL;
 }
 
 void
 evas_object_smart_member_cache_invalidate(Evas_Object *obj)
 {
+   Evas_Object_Smart *o;
    Evas_Object_List *l;
-   
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+   if (o->magic != MAGIC_OBJ_SMART)
+     return;
+
    obj->parent_cache_valid = 0;
-   for (l = obj->smart.contained; l; l = l->next)
+   for (l = o->contained; l; l = l->next)
      {
 	Evas_Object *obj2;
 	
@@ -480,6 +683,46 @@ evas_object_smart_member_cache_invalidate(Evas_Object *obj)
      }
 }
 
+void
+evas_object_smart_member_raise(Evas_Object *member)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(member->smart.parent->object_data);
+   o->contained = evas_object_list_remove(o->contained, member);
+   o->contained = evas_object_list_append(o->contained, member);
+}
+
+void
+evas_object_smart_member_lower(Evas_Object *member)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(member->smart.parent->object_data);
+   o->contained = evas_object_list_remove(o->contained, member);
+   o->contained = evas_object_list_prepend(o->contained, member);
+}
+
+void
+evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(member->smart.parent->object_data);
+   o->contained = evas_object_list_remove(o->contained, member);
+   o->contained = evas_object_list_append_relative(o->contained, member, other);
+}
+
+void
+evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(member->smart.parent->object_data);
+   o->contained = evas_object_list_remove(o->contained, member);
+   o->contained = evas_object_list_prepend_relative(o->contained, member, other);
+}
+
 /* all nice and private */
 static void
 evas_object_smart_init(Evas_Object *obj)
@@ -548,3 +791,30 @@ evas_object_smart_render_post(Evas_Object *obj)
 {
    obj->prev = obj->cur;
 }
+
+static unsigned int evas_object_smart_id_get(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_SMART;
+}
+
+static unsigned int evas_object_smart_visual_id_get(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+   if (!o) return 0;
+   return MAGIC_OBJ_CONTAINER;
+}
+
+static void *evas_object_smart_engine_data_get(Evas_Object *obj)
+{
+   Evas_Object_Smart *o;
+
+   o = (Evas_Object_Smart *)(obj->object_data);
+   if (!o) return NULL;
+   return o->engine_data;
+}
