@@ -5,82 +5,26 @@
 #define _GNU_SOURCE
 #endif
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include "Evas.h"
 
-/* the evas module api */
-/***********************/
-/* the module api version */
-#define EVAS_MODULE_API_VERSION 1
-
-/* the module types */
-typedef enum _Evas_Module_Type
-{
-   EVAS_MODULE_TYPE_ENGINE,
-   EVAS_MODULE_TYPE_IMAGE_LOADER,
-   EVAS_MODULE_TYPE_IMAGE_SAVER,
-     EVAS_MODULE_TYPE_OBJECT
-} Evas_Module_Type;
-
-/* the module api structure, all modules should define this struct */
-typedef struct _Evas_Module_Api Evas_Module_Api;
-struct _Evas_Module_Api
-{
-   int			version;
-   Evas_Module_Type	type;
-   const char 		*name;
-   const char		*author;
-};
-
-/* the module structure */
-typedef struct _Evas_Module Evas_Module;
-struct _Evas_Module
-{
-   Evas_Module_Api 	*api;	
-   void			*handle;	/* the dlopen handle */
-   char			*path;		/* the path where this modules is */
-   char			*name;		/* the name of the dir where this module is */
-   struct
-     {
-	int (*open)(Evas_Module *);
-	void (*close)(Evas_Module *);
-     } func;
-   void		*functions;	/* this are the functions exported by the module */
-   void		*data;		/* some internal data for the module i.e the id for engines */
-
-   Evas_Module_Type	type;		/* the type detected by the path */
-   
-   int           ref; /* how many refs */
-   int           last_used; /* the cycle count when it was last used */
-   
-   unsigned char	loaded : 1;
-};
-
-
-/* the internals of the module api use this struct to reference a path with a module type
- * instead of deduce the type from the path. 
- * */
-typedef struct _Evas_Module_Path Evas_Module_Path;
-struct _Evas_Module_Path
-{
-   Evas_Module_Type	type;
-   char		       *path;
-};
-
-typedef struct _Evas_Module_Engine Evas_Module_Engine;
-struct _Evas_Module_Engine
-{
-   int 			id;
-};
-
-/* end of evas module api */
-/**************************/
+#include "../file/evas_module.h"
+#include "../file/evas_path.h"
 
+#ifdef EVAS_MAGIC_DEBUG
 /* complain when peole pass in wrong object types etc. */
-#define MAGIC_DEBUG
+# define MAGIC_DEBUG
+#endif
 
 #define RENDER_METHOD_INVALID            0x00000000
 
 typedef struct _Evas_Layer                  Evas_Layer;
+typedef struct _Evas_Size                   Evas_Size;
+typedef struct _Evas_Aspect                 Evas_Aspect;
+typedef struct _Evas_Size_Hints             Evas_Size_Hints;
 typedef struct _Evas_Font_Dir               Evas_Font_Dir;
 typedef struct _Evas_Font                   Evas_Font;
 typedef struct _Evas_Font_Alias             Evas_Font_Alias;
@@ -100,6 +44,7 @@ typedef struct _Evas_Intercept_Func_Color   Evas_Intercept_Func_Color;
 typedef struct _Evas_Key_Grab               Evas_Key_Grab;
 typedef struct _Evas_Callbacks              Evas_Callbacks;
 typedef struct _Evas_Format                 Evas_Format;
+typedef struct _Evas_Rectangles             Evas_Rectangles;
 
 #define MAGIC_EVAS          0x70777770
 #define MAGIC_OBJ           0x71777770
@@ -112,22 +57,28 @@ typedef struct _Evas_Format                 Evas_Format;
 #define MAGIC_OBJ_SMART     0x71777777
 #define MAGIC_OBJ_TEXTBLOCK 0x71777778
 #define MAGIC_SMART         0x72777770
+#define MAGIC_OBJ_GRADIENT_LINEAR  0x72777771
+#define MAGIC_OBJ_GRADIENT_RADIAL  0x72777772
+#define MAGIC_OBJ_SHAPE  0x72777773
+#define MAGIC_OBJ_CONTAINER  0x72777774
+#define MAGIC_OBJ_CUSTOM  0x72777775
 
 #ifdef MAGIC_DEBUG
-#define MAGIC_CHECK_FAILED(o, t, m) \
+# define MAGIC_CHECK_FAILED(o, t, m) \
 {evas_debug_error(); \
  if (!o) evas_debug_input_null(); \
  else if (((t *)o)->magic == 0) evas_debug_magic_null(); \
  else evas_debug_magic_wrong((m), ((t *)o)->magic); \
 }
-#else
-#define MAGIC_CHECK_FAILED(o, t, m)
-#endif
-#define MAGIC_CHECK(o, t, m) \
+# define MAGIC_CHECK(o, t, m) \
 {if ((!o) || (!(((t *)o)->magic == (m)))) { \
 MAGIC_CHECK_FAILED(o, t, m)
-#define MAGIC_CHECK_END() \
-}}
+# define MAGIC_CHECK_END() }}
+#else
+# define MAGIC_CHECK_FAILED(o, t, m)
+# define MAGIC_CHECK(o, t, m)  { if (!o) {
+# define MAGIC_CHECK_END() }}
+#endif
 
 #define NEW_RECT(_r, _x, _y, _w, _h) \
 {(_r) = malloc(sizeof(Evas_Rectangle)); \
@@ -230,7 +181,7 @@ struct _Evas_Smart
 
    int               usage;
 
-   Evas_Smart_Class *smart_class;
+   const Evas_Smart_Class *smart_class;
 
    unsigned char     delete_me : 1;
    unsigned char     class_allocated : 1;
@@ -260,7 +211,7 @@ struct _Evas_Callbacks
    Evas_Object_List *callbacks;
    int               walking_list;
    unsigned char     deletions_waiting : 1;
-/*   
+/*
    Evas_Object_List *down;
    Evas_Object_List *up;
    Evas_Object_List *move;
@@ -336,6 +287,15 @@ struct _Evas
       int   info_magic;
    } engine;
 
+   Evas_Array     delete_objects;
+   Evas_Array     active_objects;
+   Evas_Array     restack_objects;
+   Evas_Array	  render_objects;
+   Evas_Array	  pending_objects;
+   Evas_Array     obscuring_objects;
+   Evas_Array     temporary_objects;
+   Evas_Array     calculate_objects;
+
    int            delete_grabs;
    int            walking_grabs;
    Evas_List     *grabs;
@@ -350,15 +310,17 @@ struct _Evas
    int            last_mouse_down_counter;
    int            last_mouse_up_counter;
    Evas_Font_Hinting_Flags hinting;
-   unsigned char     changed : 1;
+   unsigned char  changed : 1;
    unsigned char  delete_me : 1;
+   unsigned char  invalidate : 1;
+   unsigned char  cleanup : 1;
 };
 
 struct _Evas_Layer
 {
    Evas_Object_List  _list_data;
 
-   int               layer;
+   short             layer;
    Evas_Object      *objects;
 
    Evas             *evas;
@@ -368,6 +330,30 @@ struct _Evas_Layer
    unsigned char     delete_me : 1;
 };
 
+struct _Evas_Rectangles
+{
+   unsigned int    total;
+   unsigned int    count;
+   Evas_Rectangle *array;
+};
+
+struct _Evas_Size
+{
+   Evas_Coord w, h;
+};
+
+struct _Evas_Aspect
+{
+   Evas_Aspect_Control mode;
+   Evas_Size size;
+};
+
+struct _Evas_Size_Hints
+{
+   Evas_Size min, max, request;
+   Evas_Aspect aspect;
+};
+
 struct _Evas_Object
 {
    Evas_Object_List  _list_data;
@@ -379,7 +365,7 @@ struct _Evas_Object
 
    struct {
       struct {
-/*	 
+/*
 	 struct {
 	    int            x, y, w, h;
 	    int            validity;
@@ -388,25 +374,22 @@ struct _Evas_Object
 	 struct {
 	    int            x, y, w, h;
 	    unsigned char  r, g, b, a;
-	    unsigned char  visible : 1;
-	    unsigned char  dirty : 1;
+	    Evas_Bool      visible : 1;
+	    Evas_Bool      dirty : 1;
 	 } clip;
       } cache;
-      struct {
-	 Evas_Coord         x, y, w, h;
-      } geometry;
+      double scale;
+      Evas_Coord_Rectangle geometry;
       struct {
 	 unsigned char  r, g, b, a;
       } color;
-      unsigned char     visible : 1;
-      unsigned char     have_clipees : 1;
-      int               layer;
       Evas_Object      *clipper;
-      unsigned char     anti_alias;
-      struct {
-         int color_space;
-      } interpolation;
-      int               render_op;
+      short             layer;
+      Evas_Bool         visible : 1;
+      Evas_Bool         have_clipees : 1;
+      Evas_Bool         anti_alias : 1;
+      unsigned char     interpolation_color_space : 1;
+      Evas_Render_Op    render_op : 4;
    } cur, prev;
 
    char                       *name;
@@ -431,36 +414,35 @@ struct _Evas_Object
    void             *object_data;
 
    struct {
-      int               walking_list;
       Evas_Smart       *smart;
-      void             *data;
       Evas_Object      *parent;
-      Evas_Object_List *contained;
-      Evas_List        *callbacks;
-      unsigned char     deletions_waiting : 1;
    } smart;
 
+   Evas_Size_Hints            *size_hints;
+
    int                         last_mouse_down_counter;
    int                         last_mouse_up_counter;
    int                         mouse_grabbed;
 
-   unsigned short              store : 1;
-   unsigned short              pass_events : 1;
-   unsigned short              parent_pass_events : 1;
-   unsigned short              parent_cache_valid : 1;
-   unsigned short              repeat_events : 1;
-   unsigned short              restack : 1;
-   unsigned short              changed : 1;
-   unsigned short              mouse_in : 1;
-   unsigned short              pre_render_done : 1;
-   unsigned short              intercepted : 1;
-   unsigned short              focused : 1;
-   unsigned short              in_layer : 1;
-   unsigned short              no_propagate : 1;
+   Evas_Object_Pointer_Mode    pointer_mode : 1;
 
-   unsigned short              precise_is_inside : 1;
-
-   Evas_Object_Pointer_Mode    pointer_mode;
+   Evas_Bool                   store : 1;
+   Evas_Bool                   pass_events : 1;
+   Evas_Bool                   parent_pass_events : 1;
+   Evas_Bool                   parent_cache_valid : 1;
+   Evas_Bool                   repeat_events : 1;
+   Evas_Bool                   restack : 1;
+   Evas_Bool                   changed : 1;
+   Evas_Bool                   is_active : 1;
+   Evas_Bool                   render_pre : 1;
+   Evas_Bool                   rect_del : 1;
+   Evas_Bool                   mouse_in : 1;
+   Evas_Bool                   pre_render_done : 1;
+   Evas_Bool                   intercepted : 1;
+   Evas_Bool                   focused : 1;
+   Evas_Bool                   in_layer : 1;
+   Evas_Bool                   no_propagate : 1;
+   Evas_Bool                   precise_is_inside : 1;
 
    unsigned char               delete_me;
 };
@@ -515,6 +497,10 @@ struct _Evas_Object_Func
    void (*render_pre) (Evas_Object *obj);
    void (*render_post) (Evas_Object *obj);
 
+   unsigned int  (*type_id_get) (Evas_Object *obj);
+   unsigned int  (*visual_id_get) (Evas_Object *obj);
+   void *(*engine_data_get) (Evas_Object *obj);
+
    void (*store) (Evas_Object *obj);
    void (*unstore) (Evas_Object *obj);
 
@@ -575,6 +561,29 @@ struct _Evas_Func
    void *(*polygon_points_clear)           (void *data, void *context, void *polygon);
    void (*polygon_draw)                    (void *data, void *context, void *surface, void *polygon);
 
+   void (*gradient2_color_np_stop_insert)   (void *data, void *gradient, int r, int g, int b, int a, float pos);
+   void (*gradient2_clear)                  (void *data, void *gradient);
+   void (*gradient2_fill_transform_set)     (void *data, void *gradient, void *transform);
+   void (*gradient2_fill_spread_set)        (void *data, void *gradient, int spread);
+
+   void *(*gradient2_linear_new)                   (void *data);
+   void (*gradient2_linear_free)                   (void *data, void *linear_gradient);
+   void (*gradient2_linear_fill_set)               (void *data, void *linear_gradient, float x0, float y0, float x1, float y1);
+   int  (*gradient2_linear_is_opaque)              (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
+   int  (*gradient2_linear_is_visible)             (void *data, void *context, void *linear_gradient, int x, int y, int w, int h);
+   void (*gradient2_linear_render_pre)             (void *data, void *context, void *linear_gradient);
+   void (*gradient2_linear_render_post)            (void *data, void *linear_gradient);
+   void (*gradient2_linear_draw)                   (void *data, void *context, void *surface, void *linear_gradient, int x, int y, int w, int h);
+
+   void *(*gradient2_radial_new)                   (void *data);
+   void (*gradient2_radial_free)                   (void *data, void *radial_gradient);
+   void (*gradient2_radial_fill_set)               (void *data, void *radial_gradient, float cx, float cy, float rx, float ry);
+   int  (*gradient2_radial_is_opaque)              (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
+   int  (*gradient2_radial_is_visible)             (void *data, void *context, void *radial_gradient, int x, int y, int w, int h);
+   void (*gradient2_radial_render_pre)             (void *data, void *context, void *radial_gradient);
+   void (*gradient2_radial_render_post)            (void *data, void *radial_gradient);
+   void (*gradient2_radial_draw)                   (void *data, void *context, void *surface, void *radial_gradient, int x, int y, int w, int h);
+
    void *(*gradient_new)                   (void *data);
    void (*gradient_free)                   (void *data, void *gradient);
    void (*gradient_color_stop_add)         (void *data, void *gradient, int r, int g, int b, int a, int delta);
@@ -605,6 +614,8 @@ struct _Evas_Func
    void *(*image_dirty_region)             (void *data, void *image, int x, int y, int w, int h);
    void *(*image_data_get)                 (void *data, void *image, int to_write, DATA32 **image_data);
    void *(*image_data_put)                 (void *data, void *image, DATA32 *image_data);
+   void  (*image_data_preload_request)     (void *data, void *image, const void *target);
+   void  (*image_data_preload_cancel)      (void *data, void *image);
    void *(*image_alpha_set)                (void *data, void *image, int has_alpha);
    int  (*image_alpha_get)                 (void *data, void *image);
    void *(*image_border_set)               (void *data, void *image, int l, int r, int t, int b);
@@ -643,18 +654,18 @@ struct _Evas_Func
    int  (*font_cache_get)                  (void *data);
 
    /* Engine functions will over time expand from here */
-   
+
    void (*font_hinting_set)                (void *data, void *font, int hinting);
    int  (*font_hinting_can_hint)           (void *data, int hinting);
-   
+
 /*    void (*image_rotation_set)              (void *data, void *image); */
 
 };
 
 struct _Evas_Image_Load_Func
 {
-  int (*file_head) (RGBA_Image *im, const char *file, const char *key);
-  int (*file_data) (RGBA_Image *im, const char *file, const char *key);
+  int (*file_head) (Image_Entry *ie, const char *file, const char *key);
+  int (*file_data) (Image_Entry *ie, const char *file, const char *key);
 };
 
 struct _Evas_Image_Save_Func
@@ -671,44 +682,30 @@ void evas_object_free(Evas_Object *obj, int clean_layer);
 void evas_object_inject(Evas_Object *obj, Evas *e);
 void evas_object_release(Evas_Object *obj, int clean_layer);
 void evas_object_change(Evas_Object *obj);
-Evas_List *evas_object_render_pre_visible_change(Evas_List *updates, Evas_Object *obj, int is_v, int was_v);
-Evas_List *evas_object_render_pre_clipper_change(Evas_List *updates, Evas_Object *obj);
-Evas_List *evas_object_render_pre_prev_cur_add(Evas_List *updates, Evas_Object *obj);
-void evas_object_render_pre_effect_updates(Evas_List *updates, Evas_Object *obj, int is_v, int was_v);
-Evas_List * evas_rects_return_difference_rects(int x, int y, int w, int h, int xx, int yy, int ww, int hh);
-void evas_object_clip_recalc(Evas_Object *obj);
+void evas_object_render_pre_visible_change(Evas_Rectangles *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_object_render_pre_clipper_change(Evas_Rectangles *rects, Evas_Object *obj);
+void evas_object_render_pre_prev_cur_add(Evas_Rectangles *rects, Evas_Object *obj);
+void evas_object_render_pre_effect_updates(Evas_Rectangles *rects, Evas_Object *obj, int is_v, int was_v);
+void evas_rects_return_difference_rects(Evas_Rectangles *rects, int x, int y, int w, int h, int xx, int yy, int ww, int hh);
+
 void evas_object_clip_dirty(Evas_Object *obj);
 void evas_object_recalc_clippees(Evas_Object *obj);
 Evas_Layer *evas_layer_new(Evas *e);
 void evas_layer_pre_free(Evas_Layer *lay);
 void evas_layer_free(Evas_Layer *lay);
-Evas_Layer *evas_layer_find(Evas *e, int layer_num);
+Evas_Layer *evas_layer_find(Evas *e, short layer_num);
 void evas_layer_add(Evas_Layer *lay);
 void evas_layer_del(Evas_Layer *lay);
-void evas_object_coords_recalc(Evas_Object *obj);
-int evas_object_is_active(Evas_Object *obj);
-int evas_object_is_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
-int evas_object_was_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
-int evas_object_is_visible(Evas_Object *obj);
+
 int evas_object_was_visible(Evas_Object *obj);
-int evas_object_is_opaque(Evas_Object *obj);
+int evas_object_was_in_output_rect(Evas_Object *obj, int x, int y, int w, int h);
+
 int evas_object_was_opaque(Evas_Object *obj);
 int evas_object_is_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
 int evas_object_was_inside(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
-//void evas_object_recalc_clippees(Evas_Object *obj);
-int evas_object_clippers_is_visible(Evas_Object *obj);
 int evas_object_clippers_was_visible(Evas_Object *obj);
 void evas_object_event_callback_call(Evas_Object *obj, Evas_Callback_Type type, void *event_info);
-int evas_event_passes_through(Evas_Object *obj);
 Evas_List *evas_event_objects_event_list(Evas *e, Evas_Object *stop, int x, int y);
-int evas_file_path_is_full_path(const char *path);
-char *evas_file_path_join(const char *path, const char *end);
-int evas_file_path_exists(const char *path);
-int evas_file_path_is_file(const char *path);
-int evas_file_path_is_dir(const char *path);
-Evas_List *evas_file_path_list(char *path, const char *match, int match_case);
-DATA64 evas_file_modified_time(const char *file);
-char *evas_file_path_resolve(const char *file);
 int evas_mem_free(int mem_required);
 int evas_mem_degrade(int mem_required);
 void evas_debug_error(void);
@@ -721,6 +718,12 @@ void evas_object_smart_use(Evas_Smart *s);
 void evas_object_smart_unuse(Evas_Smart *s);
 void evas_object_smart_del(Evas_Object *obj);
 void evas_object_smart_cleanup(Evas_Object *obj);
+void evas_object_smart_member_raise(Evas_Object *member);
+void evas_object_smart_member_lower(Evas_Object *member);
+void evas_object_smart_member_stack_above(Evas_Object *member, Evas_Object *other);
+void evas_object_smart_member_stack_below(Evas_Object *member, Evas_Object *other);
+const Evas_Object_List *evas_object_smart_members_get_direct(const Evas_Object *obj);
+void evas_call_smarts_calculate(Evas *e);
 void *evas_mem_calloc(int size);
 void evas_object_event_callback_all_del(Evas_Object *obj);
 void evas_object_event_callback_cleanup(Evas_Object *obj);
@@ -729,6 +732,7 @@ void evas_object_inform_call_hide(Evas_Object *obj);
 void evas_object_inform_call_move(Evas_Object *obj);
 void evas_object_inform_call_resize(Evas_Object *obj);
 void evas_object_inform_call_restack(Evas_Object *obj);
+void evas_object_inform_call_changed_size_hints(Evas_Object *obj);
 void evas_object_intercept_cleanup(Evas_Object *obj);
 int evas_object_intercept_call_show(Evas_Object *obj);
 int evas_object_intercept_call_hide(Evas_Object *obj);
@@ -746,16 +750,16 @@ void evas_object_grabs_cleanup(Evas_Object *obj);
 void evas_key_grab_free(Evas_Object *obj, const char *keyname, Evas_Modifier_Mask modifiers, Evas_Modifier_Mask not_modifiers);
 void evas_font_dir_cache_free(void);
 const char *evas_font_dir_cache_find(char *dir, char *font);
-Evas_List *evas_font_dir_available_list(Evas* evas);
+Evas_List *evas_font_dir_available_list(const Evas* evas);
 void evas_font_dir_available_list_free(Evas_List *available);
 void evas_font_free(Evas *evas, void *font);
 void *evas_font_load(Evas *evas, const char *name, const char *source, int size);
-void evas_font_load_hinting_set(Evas *evas, void *font, int hinting);   
+void evas_font_load_hinting_set(Evas *evas, void *font, int hinting);
 void evas_object_smart_member_cache_invalidate(Evas_Object *obj);
 void evas_text_style_pad_get(Evas_Text_Style_Type style, int *l, int *r, int *t, int *b);
 void _evas_object_text_rehint(Evas_Object *obj);
 void _evas_object_textblock_rehint(Evas_Object *obj);
-       
+
 extern int _evas_alloc_error;
 
 struct _Evas_Imaging_Image
@@ -768,38 +772,22 @@ struct _Evas_Imaging_Font
    RGBA_Font *font;
 };
 
-typedef struct _Evas_Mempool Evas_Mempool;
-   
-struct _Evas_Mempool
-{
-   int           item_size;
-   int           pool_size;
-   int           usage;
-   void         *first, *last;
-};
-
-void *evas_mempool_malloc(Evas_Mempool *pool, int size);
-void  evas_mempool_free(Evas_Mempool *pool, void *ptr);
-void *evas_mempool_calloc(Evas_Mempool *pool, int size);
-
-void evas_module_paths_init(void);
-void evas_module_init(void);
-Evas_Module * evas_module_find_type(Evas_Module_Type type, const char *name);
-int evas_module_load(Evas_Module *em);
-void evas_module_unload(Evas_Module *em);
-void evas_module_ref(Evas_Module *em);
-void evas_module_unref(Evas_Module *em);
-void evas_module_use(Evas_Module *em);
-void evas_module_clean(void);
-void evas_module_shutdown(void);
+int evas_async_events_init(void);
+int evas_async_events_shutdown(void);
 
 void _evas_walk(Evas *e);
 void _evas_unwalk(Evas *e);
-       
+
 EAPI int _evas_module_engine_inherit(Evas_Func *funcs, char *name);
-       
+
+void evas_render_invalidate(Evas *e);
+void evas_render_object_recalc(Evas_Object *obj);
+
 #define EVAS_API_OVERRIDE(func, api, prefix) \
      (api)->func = prefix##func
+
+#include "evas_inline.x"
+
 #ifdef __cplusplus
 }
 #endif
