@@ -193,6 +193,165 @@ eng_polygon_draw(void *data, void *context, void *surface, void *polygon)
      }
 }
 
+static void
+eng_gradient2_color_np_stop_insert(void *data, void *gradient, int r, int g, int b, int a, float pos)
+{
+   evas_common_gradient2_color_np_stop_insert(gradient, r, g, b, a, pos);
+}
+
+static void
+eng_gradient2_clear(void *data, void *gradient)
+{
+   evas_common_gradient2_clear(gradient);
+}
+
+static void
+eng_gradient2_fill_transform_set(void *data, void *gradient, void *transform)
+{
+   evas_common_gradient2_fill_transform_set(gradient, transform);
+}
+
+static void
+eng_gradient2_fill_spread_set
+(void *data, void *gradient, int spread)
+{
+   evas_common_gradient2_fill_spread_set(gradient, spread);
+}
+
+static void *
+eng_gradient2_linear_new(void *data)
+{
+   return evas_common_gradient2_linear_new();
+}
+
+static void
+eng_gradient2_linear_free(void *data, void *linear_gradient)
+{
+   evas_common_gradient2_free(linear_gradient);
+}
+
+static void
+eng_gradient2_linear_fill_set(void *data, void *linear_gradient, float x0, float y0, float x1, float y1)
+{
+   evas_common_gradient2_linear_fill_set(linear_gradient, x0, y0, x1, y1);
+}
+
+static int
+eng_gradient2_linear_is_opaque(void *data, void *context, void *linear_gradient, int x, int y, int w, int h)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+   RGBA_Gradient2 *gr = (RGBA_Gradient2 *)linear_gradient;
+
+   if (!dc || !gr || !gr->type.geometer)  return 0;
+   return !(gr->type.geometer->has_alpha(gr, dc->render_op) |
+              gr->type.geometer->has_mask(gr, dc->render_op));
+}
+
+static int
+eng_gradient2_linear_is_visible(void *data, void *context, void *linear_gradient, int x, int y, int w, int h)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+
+   if (!dc || !linear_gradient)  return 0;
+   return 1;
+}
+
+static void
+eng_gradient2_linear_render_pre(void *data, void *context, void *linear_gradient)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+   RGBA_Gradient2 *gr = (RGBA_Gradient2 *)linear_gradient;
+   int  len;
+
+   if (!dc || !gr || !gr->type.geometer)  return;
+   gr->type.geometer->geom_update(gr);
+   len = gr->type.geometer->get_map_len(gr);
+   evas_common_gradient2_map(dc, gr, len);
+}
+
+static void
+eng_gradient2_linear_render_post(void *data, void *linear_gradient)
+{
+}
+
+static void
+eng_gradient2_linear_draw(void *data, void *context, void *surface, void *linear_gradient, int x, int y, int w, int h)
+{
+#ifdef BUILD_PTHREAD
+   if (cpunum > 1)
+     evas_common_pipe_grad2_draw(surface, context, x, y, w, h, linear_gradient);
+   else
+#endif
+     evas_common_gradient2_draw(surface, context, x, y, w, h, linear_gradient);
+}
+
+static void *
+eng_gradient2_radial_new(void *data)
+{
+   return evas_common_gradient2_radial_new();
+}
+
+static void
+eng_gradient2_radial_free(void *data, void *radial_gradient)
+{
+   evas_common_gradient2_free(radial_gradient);
+}
+
+static void
+eng_gradient2_radial_fill_set(void *data, void *radial_gradient, float cx, float cy, float rx, float ry)
+{
+   evas_common_gradient2_radial_fill_set(radial_gradient, cx, cy, rx, ry);
+}
+
+static int
+eng_gradient2_radial_is_opaque(void *data, void *context, void *radial_gradient, int x, int y, int w, int h)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+   RGBA_Gradient2 *gr = (RGBA_Gradient2 *)radial_gradient;
+
+   if (!dc || !gr || !gr->type.geometer)  return 0;
+   return !(gr->type.geometer->has_alpha(gr, dc->render_op) |
+              gr->type.geometer->has_mask(gr, dc->render_op));
+}
+
+static int
+eng_gradient2_radial_is_visible(void *data, void *context, void *radial_gradient, int x, int y, int w, int h)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+
+   if (!dc || !radial_gradient)  return 0;
+   return 1;
+}
+
+static void
+eng_gradient2_radial_render_pre(void *data, void *context, void *radial_gradient)
+{
+   RGBA_Draw_Context *dc = (RGBA_Draw_Context *)context;
+   RGBA_Gradient2 *gr = (RGBA_Gradient2 *)radial_gradient;
+   int  len;
+
+   if (!dc || !gr || !gr->type.geometer)  return;
+   gr->type.geometer->geom_update(gr);
+   len = gr->type.geometer->get_map_len(gr);
+   evas_common_gradient2_map(dc, gr, len);
+}
+
+static void
+eng_gradient2_radial_render_post(void *data, void *radial_gradient)
+{
+}
+
+static void
+eng_gradient2_radial_draw(void *data, void *context, void *surface, void *radial_gradient, int x, int y, int w, int h)
+{
+#ifdef BUILD_PTHREAD
+   if (cpunum > 1)
+     evas_common_pipe_grad2_draw(surface, context, x, y, w, h, radial_gradient);
+   else
+#endif
+     evas_common_gradient2_draw(surface, context, x, y, w, h, radial_gradient);
+}
+
 static void *
 eng_gradient_new(void *data)
 {
@@ -332,14 +491,14 @@ eng_gradient_draw(void *data, void *context, void *surface, void *gradient, int
 static int
 eng_image_alpha_get(void *data, void *image)
 {
-   RGBA_Image *im;
+   Image_Entry *im;
 
    if (!image) return 1;
    im = image;
-   switch (im->cs.space)
+   switch (im->space)
      {
       case EVAS_COLORSPACE_ARGB8888:
-	if (im->flags & RGBA_IMAGE_HAS_ALPHA) return 1;
+	if (im->flags.alpha) return 1;
       default:
 	break;
      }
@@ -349,11 +508,11 @@ eng_image_alpha_get(void *data, void *image)
 static int
 eng_image_colorspace_get(void *data, void *image)
 {
-   RGBA_Image *im;
+   Image_Entry *im;
 
    if (!image) return EVAS_COLORSPACE_ARGB8888;
    im = image;
-   return im->cs.space;
+   return im->space;
 }
 
 static void *
@@ -363,18 +522,15 @@ eng_image_alpha_set(void *data, void *image, int has_alpha)
 
    if (!image) return NULL;
    im = image;
-   if (im->cs.space != EVAS_COLORSPACE_ARGB8888)
+   if (im->cache_entry.space != EVAS_COLORSPACE_ARGB8888)
      {
-	im->flags &= ~RGBA_IMAGE_HAS_ALPHA;
+	im->cache_entry.flags.alpha = 0;
 	return im;
      }
-   im = evas_cache_image_alone(im);
+   im = (RGBA_Image *) evas_cache_image_alone(&im->cache_entry);
    evas_common_image_colorspace_dirty(im);
 
-   if (has_alpha)
-     im->flags |= RGBA_IMAGE_HAS_ALPHA;
-   else
-     im->flags &= ~RGBA_IMAGE_HAS_ALPHA;
+   im->cache_entry.flags.alpha = has_alpha ? 1 : 0;
    return im;
 }
 
@@ -414,42 +570,11 @@ eng_image_format_get(void *data, void *image)
 static void
 eng_image_colorspace_set(void *data, void *image, int cspace)
 {
-   RGBA_Image *im;
-   
+   Image_Entry *im;
+
    if (!image) return;
    im = image;
-   if (im->cs.space == cspace) return;
-   switch (cspace)
-     {
-      case EVAS_COLORSPACE_ARGB8888:
-	if (im->cs.data)
-	  {
-	     if (!im->cs.no_free) free(im->cs.data);
-	     im->cs.data = NULL;
-	     im->cs.no_free = 0;
-	  }
-	break;
-      case EVAS_COLORSPACE_YCBCR422P601_PL:
-      case EVAS_COLORSPACE_YCBCR422P709_PL:
-	if (im->image->no_free)
-	  {
-	     im->image->data = NULL;
-	     im->image->no_free = 0;
-	     evas_common_image_surface_alloc(im->image);
-	  }
-	if (im->cs.data)
-	  {
-	     if (!im->cs.no_free) free(im->cs.data);
-	  }
-	im->cs.data = calloc(1, im->image->h * sizeof(unsigned char *) * 2);
-	im->cs.no_free = 0;
-	break;
-      default:
-	abort();
-	break;
-     }
-   im->cs.space = cspace;
-   evas_common_image_colorspace_dirty(im);
+   evas_cache_image_colorspace(im, cspace);
 }
 
 static void
@@ -491,17 +616,17 @@ eng_image_free(void *data, void *image)
 static void
 eng_image_size_get(void *data, void *image, int *w, int *h)
 {
-   RGBA_Image *im;
+   Image_Entry *im;
 
    im = image;
-   if (w) *w = im->image->w;
-   if (h) *h = im->image->h;
+   if (w) *w = im->w;
+   if (h) *h = im->h;
 }
 
 static void *
 eng_image_size_set(void *data, void *image, int w, int h)
 {
-   RGBA_Image *im;
+   Image_Entry *im;
 
    im = image;
    return evas_cache_image_size_set(image, w, h);
@@ -510,11 +635,10 @@ eng_image_size_set(void *data, void *image, int w, int h)
 static void *
 eng_image_dirty_region(void *data, void *image, int x, int y, int w, int h)
 {
-   RGBA_Image*  im = image;
+   Image_Entry *im = image;
 
    if (!image) return NULL;
-   im = evas_cache_image_dirty(im, x, y, w, h);
-   return im;
+   return evas_cache_image_dirty(im, x, y, w, h);
 }
 
 static void *
@@ -528,13 +652,13 @@ eng_image_data_get(void *data, void *image, int to_write, DATA32 **image_data)
 	return NULL;
      }
    im = image;
-   evas_common_load_image_data_from_file(im);
-   switch (im->cs.space)
+   evas_cache_image_load_data(&im->cache_entry);
+   switch (im->cache_entry.space)
      {
       case EVAS_COLORSPACE_ARGB8888:
 	if (to_write)
-          im = evas_cache_image_alone(im);
-	*image_data = im->image->data;
+          im = (RGBA_Image *) evas_cache_image_alone(&im->cache_entry);
+	*image_data = im->image.data;
 	break;
       case EVAS_COLORSPACE_YCBCR422P601_PL:
       case EVAS_COLORSPACE_YCBCR422P709_PL:
@@ -554,19 +678,19 @@ eng_image_data_put(void *data, void *image, DATA32 *image_data)
 
    if (!image) return NULL;
    im = image;
-   switch (im->cs.space)
+   switch (im->cache_entry.space)
      {
       case EVAS_COLORSPACE_ARGB8888:
-	if (image_data != im->image->data)
+	if (image_data != im->image.data)
 	  {
 	     int w, h;
 
-	     w = im->image->w;
-	     h = im->image->h;
-	     im2 = eng_image_new_from_data(data, w, h, image_data, 
+	     w = im->cache_entry.w;
+	     h = im->cache_entry.h;
+	     im2 = eng_image_new_from_data(data, w, h, image_data,
 					   eng_image_alpha_get(data, image),
 					   eng_image_colorspace_get(data, image));
-             evas_cache_image_drop(im);
+             evas_cache_image_drop(&im->cache_entry);
 	     im = im2;
 	  }
 	break;
@@ -589,6 +713,24 @@ eng_image_data_put(void *data, void *image, DATA32 *image_data)
    return im;
 }
 
+static void
+eng_image_data_preload_request(void *data, void *image, void *target)
+{
+   RGBA_Image *im = image;
+
+   if (!im) return ;
+   evas_cache_image_preload_data(&im->cache_entry, target);
+}
+
+static void
+eng_image_data_preload_cancel(void *data, void *image)
+{
+   RGBA_Image *im = image;
+
+   if (!im) return ;
+   evas_cache_image_preload_cancel(&im->cache_entry);
+}
+
 static void
 eng_image_draw(void *data, void *context, void *surface, void *image, int src_x, int src_y, int src_w, int src_h, int dst_x, int dst_y, int dst_w, int dst_h, int smooth)
 {
@@ -596,8 +738,8 @@ eng_image_draw(void *data, void *context, void *surface, void *image, int src_x,
 
    if (!image) return;
    im = image;
-   if (im->cs.space == EVAS_COLORSPACE_ARGB8888)
-     evas_common_load_image_data_from_file(im);
+   if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
+     evas_cache_image_load_data(&im->cache_entry);
    evas_common_image_colorspace_normalize(im);
 #ifdef BUILD_PTHREAD
    if (cpunum > 1)
@@ -834,6 +976,29 @@ static Evas_Func func =
      eng_polygon_points_clear,
      eng_polygon_draw,
      /* gradient draw funcs */
+     eng_gradient2_color_np_stop_insert,
+     eng_gradient2_clear,
+     eng_gradient2_fill_transform_set,
+     eng_gradient2_fill_spread_set,
+
+     eng_gradient2_linear_new,
+     eng_gradient2_linear_free,
+     eng_gradient2_linear_fill_set,
+     eng_gradient2_linear_is_opaque,
+     eng_gradient2_linear_is_visible,
+     eng_gradient2_linear_render_pre,
+     eng_gradient2_linear_render_post,
+     eng_gradient2_linear_draw,
+
+     eng_gradient2_radial_new,
+     eng_gradient2_radial_free,
+     eng_gradient2_radial_fill_set,
+     eng_gradient2_radial_is_opaque,
+     eng_gradient2_radial_is_visible,
+     eng_gradient2_radial_render_pre,
+     eng_gradient2_radial_render_post,
+     eng_gradient2_radial_draw,
+
      eng_gradient_new,
      eng_gradient_free,
      eng_gradient_color_stop_add,
@@ -864,6 +1029,8 @@ static Evas_Func func =
      eng_image_dirty_region,
      eng_image_data_get,
      eng_image_data_put,
+     eng_image_data_preload_request,
+     eng_image_data_preload_cancel,
      eng_image_alpha_set,
      eng_image_alpha_get,
      eng_image_border_set,
