@@ -5,8 +5,8 @@
 #include <tiffio.h>
 
 
-int evas_image_load_file_head_tiff(RGBA_Image *im, const char *file, const char *key);
-int evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key);
+int evas_image_load_file_head_tiff(Image_Entry *ie, const char *file, const char *key);
+int evas_image_load_file_data_tiff(Image_Entry *ie, const char *file, const char *key);
 
 Evas_Image_Load_Func evas_image_load_tiff_func =
 {
@@ -20,7 +20,7 @@ struct TIFFRGBAImage_Extra {
    TIFFRGBAImage       rgba;
    tileContigRoutine   put_contig;
    tileSeparateRoutine put_separate;
-   RGBA_Image         *image;
+   Image_Entry        *image;
    char                pper;
    uint32              num_pixels;
    uint32              py;
@@ -69,11 +69,11 @@ raster(TIFFRGBAImage_Extra * img, uint32 * rast,
    int                 image_width, image_height;
    uint32             *pixel, pixel_value;
    int                 i, j, dy, rast_offset;
-   DATA32             *buffer_pixel, *buffer = img->image->image->data;
+   DATA32             *buffer_pixel, *buffer = evas_cache_image_pixels(img->image);
    int                 alpha_premult;
 
-   image_width = img->image->image->w;
-   image_height = img->image->image->h;
+   image_width = img->image->w;
+   image_height = img->image->h;
 
    dy = h > y ? -1 : y - h;
 
@@ -110,7 +110,7 @@ raster(TIFFRGBAImage_Extra * img, uint32 * rast,
 }
 
 int
-evas_image_load_file_head_tiff(RGBA_Image *im, const char *file, const char *key)
+evas_image_load_file_head_tiff(Image_Entry *ie, const char *file, const char *key)
 {
    char                txt[1024];
    TIFFRGBAImage       tiff_image;
@@ -157,36 +157,29 @@ evas_image_load_file_head_tiff(RGBA_Image *im, const char *file, const char *key
         return 0;
      }
    strcpy(txt, "Evas Tiff loader: cannot begin reading tiff");
-   if (!TIFFRGBAImageBegin(& tiff_image, tif, 0, txt))
+   if (!TIFFRGBAImageBegin(& tiff_image, tif, 1, txt))
      {
         TIFFClose(tif);
         return 0;
      }
 
-   if (!im->image)
-     im->image = evas_common_image_surface_new(im);
-   if (!im->image)
-     {
-        TIFFClose(tif);
-	return 0;
-     }
    if (tiff_image.alpha != EXTRASAMPLE_UNSPECIFIED)
-     im->flags |= RGBA_IMAGE_HAS_ALPHA;
+     ie->flags.alpha = 1;
    if ((tiff_image.width < 1) || (tiff_image.height < 1) ||
        (tiff_image.width > 8192) || (tiff_image.height > 8192))
      {
 	TIFFClose(tif);
 	return 0;
      }
-   im->image->w = tiff_image.width;
-   im->image->h = tiff_image.height;
+   ie->w = tiff_image.width;
+   ie->h = tiff_image.height;
 
    TIFFClose(tif);
    return 1;
 }
 
 int
-evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key)
+evas_image_load_file_data_tiff(Image_Entry *ie, const char *file, const char *key)
 {
    char                txt[1024];
    TIFFRGBAImage_Extra rgba_image;
@@ -236,38 +229,28 @@ evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key
         TIFFClose(tif);
         return 0;
      }
-   rgba_image.image = im;
+   rgba_image.image = ie;
 
-   if (!im->image)
-     im->image = evas_common_image_surface_new(im);
-   if (!im->image)
-     {
-        TIFFClose(tif);
-	return 0;
-     }
    if (rgba_image.rgba.alpha != EXTRASAMPLE_UNSPECIFIED)
-     im->flags |= RGBA_IMAGE_HAS_ALPHA;
-   if ((rgba_image.rgba.width != im->image->w) ||
-       (rgba_image.rgba.height != im->image->h))
+     ie->flags.alpha = 1;
+   if ((rgba_image.rgba.width != ie->w) ||
+       (rgba_image.rgba.height != ie->h))
      {
         TIFFClose(tif);
 	return 0;
      }
-   im->image->w = rgba_image.rgba.width;
-   im->image->h = rgba_image.rgba.height;
-   rgba_image.num_pixels = num_pixels = im->image->w * im->image->h;
 
-   evas_common_image_surface_alloc(im->image);
-   if (!im->image->data)
+   evas_cache_image_surface_alloc(ie, rgba_image.rgba.width, rgba_image.rgba.height);
+   if (!evas_cache_image_pixels(ie))
      {
-	evas_common_image_surface_free(im->image);
-
         TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
         TIFFClose(tif);
 
 	return 0;
      }
 
+   rgba_image.num_pixels = num_pixels = ie->w * ie->h;
+
    rgba_image.pper = rgba_image.py = 0;
    rast = (uint32 *) _TIFFmalloc(sizeof(uint32) * num_pixels);
 
@@ -285,7 +268,6 @@ evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key
      {
         fprintf(stderr, "Evas Tiff loader: no put function");
 
-	evas_common_image_surface_free(im->image);
         _TIFFfree(rast);
         TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
         TIFFClose(tif);
@@ -305,17 +287,16 @@ evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key
             rgba_image.rgba.put.separate = put_separate_and_raster;
          }
      }
+
    /*	if (rgba_image.rgba.samplesperpixel == 8)*/
    if (rgba_image.rgba.bitspersample == 8)
      {
-        if (!TIFFRGBAImageGet((TIFFRGBAImage *) & rgba_image, rast,
+        if (!TIFFRGBAImageGet((TIFFRGBAImage *) &rgba_image, rast,
                               rgba_image.rgba.width, rgba_image.rgba.height))
           {
-             evas_common_image_surface_free(im->image);
              _TIFFfree(rast);
              TIFFRGBAImageEnd((TIFFRGBAImage *) & rgba_image);
              TIFFClose(tif);
-
              return 0;
           }
      }
@@ -330,7 +311,7 @@ evas_image_load_file_data_tiff(RGBA_Image *im, const char *file, const char *key
 
    TIFFClose(tif);
 
-   evas_common_image_set_alpha_sparse(im);
+   evas_common_image_set_alpha_sparse(ie);
    return 1;
 }
 
